/* debugmon.c, Boone, 09/29/04
   Serial port monitor for debugging
   Copyright (C) 2004-2009, Dennis Boone, East Lansing, MI */

/* Modifications:
   09/29/04 Boone      Initial coding
   End Modifications */

/* This file is part of xcatctl.

   Xcatctl is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.

   Xcatctl is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with xcatctl; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <sys/select.h>
#include "xcatctl.h"

int serfh = -2;

int seropen(char *devname, int baudrate)
{
	int rc;
	struct termios tp;

	serfh = open(devname, O_RDWR|O_NOCTTY, 0);
	if (serfh == -1)
		return(errno);

	rc = tcgetattr(serfh, &tp);
	if (rc) return(errno);
	tp.c_iflag = (IGNBRK|IGNPAR);
	tp.c_oflag = 0;
	tp.c_cflag = (CS8|CREAD|HUPCL|CLOCAL);
	tp.c_lflag = 0;
	rc = cfsetispeed(&tp, baudrate);
	if (rc) return(errno);
	rc = cfsetospeed(&tp, baudrate);
	if (rc) return(errno);
/*	cfmakeraw(&tp); */
	rc = tcsetattr(serfh, 0, &tp);
	if (rc)
		return(errno);
	return(0);
}

int serclose()
{
	if (serfh > -1)
		close(serfh);
}

int rawcharin(unsigned char *c)
{
	fd_set serset;
	struct timeval tv;
	int nd;
	ssize_t nc;

	FD_ZERO(&serset);
	FD_SET(serfh, &serset);
	tv.tv_sec = 5;
	tv.tv_usec = 0;
	nd = select(serfh+1, &serset, NULL, NULL, &tv);
	if (nd == 0)
	{
		errno = ETIMEDOUT;
		return(ETIMEDOUT);
	}

	nc = read(serfh, c, 1);

	if (nc == -1)
		return(errno);
	if (nc == 0)
	{
		errno = EPIPE;
		return(EPIPE);
	}
	return(0);
}

int main(int argc, char *argv[])
{
	int i;
	int serflag = 0;
	char serdev[1024];
	int rc;
	unsigned char c;

	for (i = 1; i < argc; i++)
	{
		if (! serflag)
		{
			strncpy(serdev, argv[i], 1023);
			serflag = 1;
			continue;
		}
		fprintf(stderr, "usage: %s serialdev\n", argv[0]);
		exit(EINVAL);
	}

	rc = seropen(serdev, B19200);
	if (rc)
	{
		perror("seropen");
		exit(rc);
	}

	while (1)
	{
		rc = rawcharin(&c);
		if (rc == ETIMEDOUT)
			continue;
		printf("%02x ", c);
		if (c == 0xfd)
			fputs("\n", stdout);
	}

	serclose();

	exit(0);
}
