/* str.c, Boone, 09/29/04
   String mangling functions
   Copyright (C) 2004-2009, Dennis Boone, East Lansing, MI */

/* Modifications:
   09/29/04 Boone      Initial coding
   End Modifications */

/* This file is part of xcatctl.

   Xcatctl is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.

   Xcatctl is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with xcatctl; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#include <stdio.h>
#include <string.h>
#include "xcatctl.h"

void bufrev(unsigned char *a, int buflen)
{
	unsigned char t;
	unsigned char *x, *y;

	x = a;
	y = a + (buflen - 1);
	while (y > a)
	{
		t = *x;
		*x = *y;
		*y = t;
		x++;
		y--;
	}
}

void tobcd(long value, unsigned char *outbuf, int buflen)
{
	long rem;
	unsigned char *o;
	int oddeven = 1;

	for (o = outbuf; o < (outbuf+buflen); o++)
		*o = 0;

	o = outbuf;
	if (debug)
		fprintf(stderr, "tobcd: o %p\n", o);
	while (value)
	{
		if (debug)
			fprintf(stderr, "tobcd: value %ld\n", value);
		rem = value % 10;
		if (debug)
			fprintf(stderr, "tobcd: rem %ld\n", rem);
		if (!oddeven)
			rem <<= 4;
		if (debug)
			fprintf(stderr, "tobcd: after oddeven shift oddeven %d rem %02x\n",
				oddeven, rem);
		rem &= 0xff;
		if (debug)
			fprintf(stderr, "tobcd: after mask rem %02x\n", rem);
		*o |= rem;
		if (debug)
			fprintf(stderr, "tobcd: after insert o %p *o %02x\n", o, *o);
		oddeven--;
		if (oddeven < 0)
		{
			o++;
			oddeven = 1;
			if (debug)
				fprintf(stderr, "tobcd: after recycling oddeven o %p\n", 0);
		}
		if (o >= (outbuf+buflen))
		{
			if (debug)
				fprintf(stderr, "tobcd: no more room in outbuf\n");
			break;
		}
		value /= 10;
		if (debug)
			fprintf(stderr, "tobcd: after div10 value %ld\n");
	}
	bufrev(outbuf, buflen);
}

void tobcdrev(long value, unsigned char *outbuf, int buflen)
{
	long rem;
	unsigned char *o;
	int oddeven = 1;

	for (o = outbuf; o < (outbuf+buflen); o++)
		*o = 0;

	o = outbuf;
	while (value)
	{
		rem = value % 10;
		if (!oddeven)
			rem <<= 4;
		rem &= 0xff;
		*o |= rem;
		oddeven--;
		if (oddeven < 0)
		{
			o++;
			oddeven = 1;
		}
		if (o >= (outbuf+buflen))
			break;
		value /= 10;
	}
}
