/* test.c, Boone, 09/29/04
   Code for testing parts of xcatctl
   Copyright (C) 2004-2009, Dennis Boone, East Lansing, MI */

/* Modifications:
   09/29/04 Boone      Initial coding
   End Modifications */

/* This file is part of xcatctl.

   Xcatctl is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2 of the License,
   or (at your option) any later version.

   Xcatctl is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with xcatctl; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA */

#define INSTANTIATE

#include <stdio.h>
#include <termios.h>
#include <stdlib.h>
#include "xcatctl.h"

#define TESTCMD

#ifdef TESTIO

/* Run this against a serial port with a modem that echoes bytes back */

int main(int argc, char *argv[])
{
	unsigned char getfwvers[] = { 0xfe, 0xfe, 0x20, 0x91, 0xaa, 0x02, 0xfd };
	int i;
	int rc;
	unsigned char c;

	rc = seropen("/dev/usb/ttyUSB1");
	if (rc)
	{
		fprintf(stderr, "rc=%d from seropen\n", rc);
		exit(0);
	}

	for (i = 0; i < 7; i++)
	{
		rc = charout(getfwvers[i]);
		if (rc)
		{
			fprintf(stderr, "\nrc=%d from charout, i=%d, c=%02x\n",
				rc, i, getfwvers[i]);
			exit(0);
		}
		printf(">> %02x ", getfwvers[i]);
	}
	puts("");

	c = 0x00;
	while (c != 0xfd)
	{
		rc = charin(&c);
		if (rc)
		{
			fprintf(stderr, "\nrc=%d from charin, c=%02x\n",
				rc, c);
			exit(0);
		}
		printf("<< %02x ", c);
	}
	puts("");
}

#endif

#ifdef TESTCMD

/* This should fetch the version stamp from your xcat */

int main(int argc, char *argv[])
{
	unsigned char getfwvers[] = { 0xaa, 0x02 };
	int rc;
	int to;
	int from;
	unsigned char ansbuf[32];
	int buflen;
	int i;

	rc = seropen("/dev/usb/ttyUSB1", B19200);
	if (rc)
	{
		fprintf(stderr, "rc=%d from seropen\n", rc);
		exit(0);
	}

	rc = sendcmd(0x20, 0x91, getfwvers, 2);
	if (rc)
	{
		fprintf(stderr, "rc %d from sendcmd\n", rc);
		exit(1);
	}

	rc = rcvresp(&to, &from, ansbuf, 32, &buflen);
	if (rc)
	{
		fprintf(stderr, "rc %d from rcvresp\n", rc);
		exit(1);
	}

	for (i = 0; i < buflen; i++)
	{
		fprintf(stderr, "<< %02x ", ansbuf[i]);
	}
	puts("");
}

#endif
